@@ -6,12 +6,12 @@
  *
  * requires GKrellM 2 and XMMS2
  *
- * Version: 0.6.4
+ * Version: 0.7.0
  *
  * Released under the GNU General Public License v2
  */
 
-#define VERSION_NUMBER "0.6.4"
+#define VERSION_NUMBER "0.7.0"
 
 #if !defined(WIN32)
 #include <gkrellm2/gkrellm.h>
@@ -62,6 +62,7 @@ static struct Configuration {
 	gboolean medialib_playlist_column_id,      medialib_playlist_column_artist;
 	gboolean medialib_playlist_column_track,   medialib_playlist_column_album;
 	gboolean medialib_playlist_column_rating,  medialib_playlist_column_pos;
+	gboolean medialib_playlist_new_confirm;
 	gboolean auto_reconnect;
 	gchar    ipc_path[256];
 } config;
@@ -95,6 +96,7 @@ static void cb_menu_playlist(GtkWidget *widget, gpointer pt)
 {
 	if (!GTK_IS_WIDGET(gml)) {
 		gml = gmedialib_new(TRUE, TRUE, local_xmms2d, TRUE, TRUE);
+		gml_pl_set_new_playlist_confirm(GMEDIALIB(gml), config.medialib_playlist_new_confirm);
 		gtk_window_resize(GTK_WINDOW(gml), config.medialib_window_width, config.medialib_window_height);
 		if (gml_notebook_is_mlib_search_visible(GMEDIALIB(gml))) {
 			gml_search_set_column_id_visible(GMEDIALIB(gml),      config.medialib_search_column_id);
@@ -533,7 +535,7 @@ static gchar *plugin_info_text[] =
 {
 "<b>GkrellM XMMS2 Control (gkrellxmms2)\n",
 "Version: ", VERSION_NUMBER, "\n",
-"gkrellxmms2 Copyright (c) 2005-2006 Johannes Heimansberg\n",
+"gkrellxmms2 Copyright (c) 2005-2007 Johannes Heimansberg\n",
 "http://wejp.k.vu/\n\n",
 "<b>Usage:\n",
 "gkrellxmms2 has five buttons to control the XMMS2 audio player.\n",
@@ -568,7 +570,7 @@ static void create_plugin_tab(GtkWidget *tab_vbox)
 	about_text = g_strdup_printf(
 		"gkrellxmms2 %s\n"
 		"GKrellM XMMS2 Control Plugin\n\n"
-		"Copyright (c) 2005-2006 Johannes Heimansberg\n"
+		"Copyright (c) 2005-2007 Johannes Heimansberg\n"
 		"http://wejp.k.vu/\n\n"
 		"Released under the GNU General Public License v2.0", 
 		VERSION_NUMBER);
@@ -587,6 +589,7 @@ static void save_config(FILE *f)
 		config.medialib_window_height           = gml_window_get_height(GMEDIALIB(gml));
 		config.medialib_window_pos_x            = gml_window_get_pos_x(GMEDIALIB(gml));
 		config.medialib_window_pos_y            = gml_window_get_pos_y(GMEDIALIB(gml));
+		config.medialib_playlist_new_confirm    = gml_pl_get_new_playlist_confirm(GMEDIALIB(gml));
 		if (gml_notebook_is_mlib_search_visible(GMEDIALIB(gml))) {
 			config.medialib_search_column_id        = gml_search_get_column_id_visible(GMEDIALIB(gml));
 			config.medialib_search_column_artist    = gml_search_get_column_artist_visible(GMEDIALIB(gml));
@@ -635,6 +638,8 @@ static void save_config(FILE *f)
 	        config.medialib_playlist_column_rating);
 	fprintf(f, "%s medialib_playlist_column_pos %d\n",     PLUGIN_NAME,
 	        config.medialib_playlist_column_pos);
+	fprintf(f, "%s medialib_playlist_new_confirm %d\n",    PLUGIN_NAME,
+	        config.medialib_playlist_new_confirm);
 	fprintf(f, "%s ipc_path %s\n",                         PLUGIN_NAME,
 	        config.ipc_path);
 	fprintf(f, "%s auto_reconnect %d\n",                   PLUGIN_NAME,
@@ -682,6 +687,8 @@ static void load_config(gchar *arg)
 			config.medialib_playlist_column_rating = atoi(item);
 		if (strcmp(cfg, "medialib_playlist_column_pos") == 0)
 			config.medialib_playlist_column_pos = atoi(item);
+		if (strcmp(cfg, "medialib_playlist_new_confirm") == 0)
+			config.medialib_playlist_new_confirm = atoi(item);
 		if (strcmp(cfg, "ipc_path") == 0)
 			g_strlcpy(config.ipc_path, item, sizeof(config.ipc_path));
 		if (strcmp(cfg, "auto_reconnect") == 0)
@@ -741,6 +748,7 @@ GkrellmMonitor * gkrellm_init_plugin(void)
 	config.medialib_playlist_column_album   = TRUE;
 	config.medialib_playlist_column_rating  = TRUE;
 	config.medialib_playlist_column_pos     = TRUE;
+	config.medialib_playlist_new_confirm    = TRUE;
 	config.auto_reconnect                   = FALSE;
 
 	username = g_get_user_name();
