@@ -2,7 +2,7 @@
  * File: gmlplaylist.c
  *
  * Medialib search and playlist editor
- * Copyright (c) 2005-2006 Johannes Heimansberg
+ * Copyright (c) 2005-2007 Johannes Heimansberg
  *
  * requires GTK+ 2.6 or better and XMMS2
  *
@@ -15,6 +15,7 @@
 #include <gtk/gtksignal.h>
 #include <gtk/gtkdialog.h>
 #include <gdk/gdkkeysyms.h>
+#include "xmmsclient/xmmsclient.h"
 #include "gmedialib.h"
 #include "gtrackinfo/trackinfo.h"
 #include "gtrackinfo/gtrackinfo.h"
@@ -29,6 +30,7 @@
 #include "rank4.xpm"
 
 extern xmmsc_connection_t *connection; /* available from gxmms2.c */
+static GdkAtom drag_atom;
 
 static void update_pl_entry(xmmsc_result_t *res,  void   *gmlptr, 
                             GtkTreeIter    *iter, gint32  pos)
@@ -44,14 +46,14 @@ static void update_pl_entry(xmmsc_result_t *res,  void   *gmlptr,
 	gint          tracknr = -1, rating = -1;
 	static gchar  last_title[256] = "", last_artist[256] = "";
 
-	xmmsc_result_get_dict_entry_str  (res,   "artist",  &artist_tmp);
-	xmmsc_result_get_dict_entry_str  (res,   "title",   &title_tmp);
-	xmmsc_result_get_dict_entry_str  (res,   "album",   &album);
-	xmmsc_result_get_dict_entry_int32(res,   "id",      &id);
-	xmmsc_result_get_dict_entry_str  (res,   "url",     &url);
-	xmmsc_result_get_dict_entry_int32(res,   "tracknr", &tracknr);
-	xmmsc_result_get_dict_entry_int32(res,   "rating",  &rating);
-	if (xmmsc_result_get_dict_entry_str(res, "channel", &channel)) {
+	xmmsc_result_get_dict_entry_string(res, "artist",  &artist_tmp);
+	xmmsc_result_get_dict_entry_string(res, "title",   &title_tmp);
+	xmmsc_result_get_dict_entry_string(res, "album",   &album);
+	xmmsc_result_get_dict_entry_int(res,    "id",      &id);
+	xmmsc_result_get_dict_entry_string(res, "url",     &url);
+	xmmsc_result_get_dict_entry_int(res,    "tracknr", &tracknr);
+	xmmsc_result_get_dict_entry_int(res,    "rating",  &rating);
+	if (xmmsc_result_get_dict_entry_string(res, "channel", &channel)) {
 		prefix = "[Stream] ";
 		album = channel;
 	} else if (album == NULL)
@@ -96,22 +98,23 @@ static void update_pl_entry(xmmsc_result_t *res,  void   *gmlptr,
 		g_strlcpy(title, title_tmp, sizeof(gchar) * strlen(title_tmp) + 1);		
 	}
 
-	if (gml->playlist.ls != NULL && 
+	if (gml->playlist.ls != NULL &&
 	    ( ( ((strcmp(title, last_title) != 0 || strcmp(artist, last_artist) != 0)) && 
 	        (title[0] != '\0' || artist[0] != '\0') ) || pos != last_pos) ) {
 		last_pos = pos;
 		g_strlcpy(last_title, title, sizeof(last_title));
 		g_strlcpy(last_artist, artist, sizeof(last_artist));
-		gtk_list_store_set(gml->playlist.ls, iter, 
-		                   1, id,
-		                   2, artist, 3, title,
-		                   4, album_with_tracknr, -1);
-		if (rating >= 0 && rating <= 4) {
+		if (gtk_list_store_iter_is_valid(gml->playlist.ls, iter)) {
 			gtk_list_store_set(gml->playlist.ls, iter, 
-			                   5, gml->playlist.icon_rating[rating], -1);
-		} else {
-			gtk_list_store_set(gml->playlist.ls, iter, 
-			                   5, NULL, -1);
+			                   1, id,
+			                   2, artist, 3, title,
+			                   4, album_with_tracknr, -1);
+			if (rating >= 0 && rating <= 4) {
+				gtk_list_store_set(gml->playlist.ls, iter, 
+				                   5, gml->playlist.icon_rating[rating], -1);
+			} else {
+				gtk_list_store_set(gml->playlist.ls, iter, 5, NULL, -1);
+			}
 		}
 	}
 	g_free(artist);
@@ -132,12 +135,12 @@ void gml_playlist_n_add_track_with_info(xmmsc_result_t *res2, void *gmlptr)
 
 	pos = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(gml->playlist.ls), NULL);
 
-	xmmsc_result_get_dict_entry_int32(res2, "id", &id);
+	xmmsc_result_get_dict_entry_int(res2, "id", &id);
 
 	if (gml->playlist.ls != NULL) {
 		gtk_list_store_append(gml->playlist.ls, &iter);
 		gtk_list_store_set(gml->playlist.ls, &iter, 
-		                   0, pos,    1, id, -1);
+		                   0, pos, 1, id, -1);
 	}
 
 	update_pl_entry(res2, gml, &iter, pos);
@@ -153,7 +156,7 @@ static void n_pl_playlist_list(xmmsc_result_t *res, void *userdata)
 	static const char *pref[] = { "client/generic", "server", "plugins/*",
 	                              "client/*", "*", NULL };
 
-	if (!gml->playlist.refresh && gml->playlist.refresh_counter == 0) {
+	if (gml->playlist.refresh_counter == 0) {
 		gml->playlist.refresh = TRUE;
 		for (; xmmsc_result_list_valid(res); xmmsc_result_list_next(res)) {
 			xmmsc_result_t *res2;
@@ -171,8 +174,6 @@ static void n_pl_playlist_list(xmmsc_result_t *res, void *userdata)
 	}
 	xmmsc_result_unref(res);
 	gml_set_statusbar_text(gml, "Ready.");
-	gtk_widget_set_sensitive(gml->playlist.button_up,   FALSE);
-	gtk_widget_set_sensitive(gml->playlist.button_down, FALSE);
 }
 
 void gml_pl_refresh_playlist(GMedialib *gml)
@@ -186,7 +187,8 @@ void gml_pl_refresh_playlist(GMedialib *gml)
 			gtk_list_store_clear(gml->playlist.ls);
 		}
 
-		res = xmmsc_playlist_list(connection);
+		res = xmmsc_playlist_list_entries(connection, "_active");
+
 		xmmsc_result_notifier_set(res, n_pl_playlist_list, gml);
 		xmmsc_result_unref(res);
 	}
@@ -209,7 +211,7 @@ static void bc_handle_current_id(xmmsc_result_t *res, void *userdata)
 {
 	xmmsc_result_t *res2;
 
-	res2 = xmmsc_playlist_current_pos(connection);
+	res2 = xmmsc_playlist_current_pos(connection, NULL);
 	xmmsc_result_notifier_set(res2, n_pl_update_current_pos, userdata);
 	xmmsc_result_unref(res2);
 }
@@ -231,64 +233,67 @@ static void bc_handle_playlist_changed(xmmsc_result_t *res, void *gmlptr)
 	GtkTreeModel *model;
 
 	/* get playlist length */
-	pl_length = 
-		gtk_tree_model_iter_n_children(GTK_TREE_MODEL(gml->playlist.ls),
-		                               NULL);
+	pl_length = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(gml->playlist.ls),
+		                                       NULL);
 
-	xmmsc_result_get_dict_entry_int32(res, "type", &signal);
+	xmmsc_result_get_dict_entry_int(res, "type", &signal);
 	switch (signal) {
 		case XMMS_PLAYLIST_CHANGED_ADD:
-			xmmsc_result_get_dict_entry_uint32(res, "id", &id);
+			xmmsc_result_get_dict_entry_uint(res, "id", &id);
 
 			/* add item to gtk list */
 			if (gml->playlist.ls != NULL) {
 				gtk_list_store_append(gml->playlist.ls, &iter);
 				gtk_list_store_set(gml->playlist.ls, &iter, 
-			   	                0, pl_length, 1, id, -1);
+				                   0, pl_length, 1, id, -1);
 				gml_playlist_update_entry(gml, pl_length);
 			}
 			break;
 		case XMMS_PLAYLIST_CHANGED_INSERT:
-			xmmsc_result_get_dict_entry_uint32(res, "id", &id);
-			xmmsc_result_get_dict_entry_int32(res, "position", &pos_ins);
-			gtk_list_store_insert(gml->playlist.ls, &iter, pos_ins);
-			gtk_list_store_set(gml->playlist.ls, &iter, 
-			   	             0, pos_ins, 1, id, -1);
-			gml_playlist_update_entry(gml, pos_ins);
-			counter = pos_ins;
-			model = GTK_TREE_MODEL(gml->playlist.ls);
-			do {
-				gtk_tree_model_get(model, &iter, 0, &pos, -1);
+			xmmsc_result_get_dict_entry_uint(res, "id", &id);
+			xmmsc_result_get_dict_entry_int(res, "position", &pos_ins);
+			if (gml->playlist.ls != NULL) {
+				gtk_list_store_insert(gml->playlist.ls, &iter, pos_ins);
 				gtk_list_store_set(gml->playlist.ls, &iter, 
-				   	             0, counter, -1);
-				counter++;
-			} while (gtk_tree_model_iter_next(model, &iter));
+				                   0, pos_ins, 1, id, -1);
+				gml_playlist_update_entry(gml, pos_ins);
+				counter = pos_ins;
+				model = GTK_TREE_MODEL(gml->playlist.ls);
+				do {
+					gtk_tree_model_get(model, &iter, 0, &pos, -1);
+					gtk_list_store_set(gml->playlist.ls, &iter, 
+					   	               0, counter, -1);
+					counter++;
+				} while (gtk_tree_model_iter_next(model, &iter));
+			}
 			break;
 		case XMMS_PLAYLIST_CHANGED_SHUFFLE:
 		case XMMS_PLAYLIST_CHANGED_SORT:
 			gml_pl_refresh_playlist(gml);
 			break;
 		case XMMS_PLAYLIST_CHANGED_REMOVE:
-			xmmsc_result_get_dict_entry_int32(res, "position", &pos_del);
-			model = GTK_TREE_MODEL(gml->playlist.ls);
+			xmmsc_result_get_dict_entry_int(res, "position", &pos_del);
+			if (gml->playlist.ls != NULL) {
+				model = GTK_TREE_MODEL(gml->playlist.ls);
 
-			if (gtk_tree_model_get_iter_first(model, &iter)) {
-				gboolean flag = FALSE;
-				counter = 0;
-				do {
-					gtk_tree_model_get(model, &iter, 0, &pos, -1);
-					if (pos == pos_del) {
-						flag = TRUE;
-						gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
-						counter = pos;
-						if (pos == pl_length - 1) break;
-					}
-					if (flag) {
-						gtk_list_store_set(gml->playlist.ls, &iter, 
-						   	             0, counter, -1);
-						counter++;
-					}
-				} while (gtk_tree_model_iter_next(model, &iter));
+				if (gtk_tree_model_get_iter_first(model, &iter)) {
+					gboolean flag = FALSE;
+					counter = 0;
+					do {
+						gtk_tree_model_get(model, &iter, 0, &pos, -1);
+						if (pos == pos_del) {
+							flag = TRUE;
+							gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
+							counter = pos;
+							if (pos == pl_length - 1) break;
+						}
+						if (flag) {
+							gtk_list_store_set(gml->playlist.ls, &iter, 
+							                   0, counter, -1);
+							counter++;
+						}
+					} while (gtk_tree_model_iter_next(model, &iter));
+				}
 			}
 			break;
 		case XMMS_PLAYLIST_CHANGED_CLEAR:
@@ -333,7 +338,7 @@ static void update_current_pl_pos(GMedialib *gml)
 {
 	xmmsc_result_t *res;
 
-	res = xmmsc_playlist_current_pos(connection);
+	res = xmmsc_playlist_current_pos(connection, NULL);
 	xmmsc_result_notifier_set(res, n_pl_update_current_pos, gml);
 	xmmsc_result_unref(res);
 }
@@ -412,23 +417,6 @@ static void cb_pl_on_tree_view_row_activated(GtkTreeView       *view,
 	}
 }
 
-static void cb_pl_on_tree_selection_changed(GtkTreeSelection *sel, 
-                                            gpointer          userdata)
-{
-	GMedialib *gml = (GMedialib *)userdata;
-	guint      cnt = 0;
-	
-	cnt = gtk_tree_selection_count_selected_rows(sel);
-
-	if (cnt == 1) {
-		gtk_widget_set_sensitive(gml->playlist.button_up,   TRUE);
-		gtk_widget_set_sensitive(gml->playlist.button_down, TRUE);
-	} else {
-		gtk_widget_set_sensitive(gml->playlist.button_up,   FALSE);
-		gtk_widget_set_sensitive(gml->playlist.button_down, FALSE);
-	}
-}
-
 enum { UP = -1, DOWN = 1 };
 
 struct params_2
@@ -496,7 +484,7 @@ static void n_playlist_move(xmmsc_result_t *res, void *arguments)
 	                       5, icon_other_rank,
 	                       -1);
 
-   		gtk_list_store_set(p->gml->playlist.ls,
+		gtk_list_store_set(p->gml->playlist.ls,
 		                   &other_row,
 		                   0, other_pos,
 		                   1, id,
@@ -554,22 +542,12 @@ static void pl_move_item(GMedialib *gml, guint direction)
 		g_list_foreach(list, (GFunc)gtk_tree_path_free, NULL);
 		g_list_free(list);
 
-		res = xmmsc_playlist_move(connection, pos, pos + direction);
+		res = xmmsc_playlist_move_entry(connection, NULL, pos, pos + direction);
 		xmmsc_result_notifier_set(res, n_playlist_move, p);
 		xmmsc_result_unref(res);
 	}
 }
 
-static void cb_pl_button_down_pressed(GtkWidget *widget, gpointer userdata)
-{
-	pl_move_item((GMedialib *)userdata, DOWN);
-}
-
-static void cb_pl_button_up_pressed(GtkWidget *widget, gpointer userdata)
-{
-	pl_move_item((GMedialib *)userdata, UP);
-}
-
 static void n_shuffle_or_sort_ready(xmmsc_result_t *res, void *userdata)
 {
 	GMedialib *gml = (GMedialib *)userdata;
@@ -584,47 +562,61 @@ static void cb_pl_shuffle_button_pressed(GtkWidget *widget, gpointer userdata)
 	xmmsc_result_t *res;
 
 	if (gml->playlist.refresh_counter == 0) {
-		res = xmmsc_playlist_shuffle(connection);
+		res = xmmsc_playlist_shuffle(connection, NULL);
 		xmmsc_result_notifier_set(res, n_shuffle_or_sort_ready, userdata);
 		xmmsc_result_unref(res);
 	}
 }
 
-static void cb_pl_sort_button_pressed(GtkWidget *widget, gpointer userdata)
+static void pl_sort_menu_pos_func(GtkMenu  *menu,
+                                  gint     *x,
+                                  gint     *y,
+                                  gboolean *push_in,
+                                  gpointer  userdata)
 {
-	xmmsc_result_t *res;
-	
-	res = xmmsc_playlist_sort(connection, "artist");
-	xmmsc_result_notifier_set(res, n_shuffle_or_sort_ready, userdata);
-	xmmsc_result_unref(res);
+	GMedialib     *gml = (GMedialib *)userdata;
+	GtkAllocation  allocation = gml->playlist.button_sort_pl->allocation;
+
+	gdk_window_get_origin(gml->playlist.button_sort_pl->window, x, y);
+	*x += allocation.x;
+	*y += allocation.y + allocation.height;
 }
 
-static void n_query_finished(xmmsc_result_t *res, void *arguments)
+static gboolean cb_pl_sort_button_raw_pressed(GtkWidget      *widget,
+                                              GdkEventButton *event,
+                                              gpointer        userdata)
 {
-	GMedialib      *gml = (GMedialib *)arguments;
-	xmmsc_result_t *res2;
-	guint           pl_length = 0;
+	GMedialib *gml = (GMedialib *)userdata;
+	gboolean   res = FALSE;
 
-	if (xmmsc_result_iserror(res)) {
-		printf("%s\n", xmmsc_result_get_error(res));
-		return;
+	if (event->button == 1) {
+		gtk_menu_popup(
+			GTK_MENU(gml->playlist.menu_sort_pl),
+			NULL,
+			NULL,
+			(GtkMenuPositionFunc) pl_sort_menu_pos_func,
+			userdata,
+			3,
+			event->time);
+		res = TRUE;
 	}
+	return res;
+}
 
-	for (; xmmsc_result_list_valid(res); xmmsc_result_list_next(res)) {
-		gint32 id;
-		xmmsc_result_get_dict_entry_int32(res, "id", &id);
+static void cb_pl_sort_menu_activate(GtkMenuItem *menuitem, gpointer userdata)
+{
+	xmmsc_result_t *res;
+	const gchar    *sortby[2];
 
-		/* get playlist length */
-		pl_length = 
-			gtk_tree_model_iter_n_children(GTK_TREE_MODEL(gml->playlist.ls),
-			                               NULL);
+	sortby[0] = (const gchar*)g_utf8_strdown(
+	                          gtk_label_get_label(GTK_LABEL(
+	                          gtk_bin_get_child(GTK_BIN(menuitem)))), -1);
+	sortby[1] = 0;
 
-		/* add id to playlist */
-		res2 = xmmsc_playlist_add_id(connection, id);
-		xmmsc_result_unref(res2);
-	}
+	res = xmmsc_playlist_sort(connection, NULL, sortby);
+	g_free((gpointer)sortby[0]);
+	xmmsc_result_notifier_set(res, n_shuffle_or_sort_ready, userdata);
 	xmmsc_result_unref(res);
-	gml_set_statusbar_text(gml, "Ready.");
 }
 
 struct params_entry_update {
@@ -636,7 +628,7 @@ struct params_entry_update {
 static void n_update_playlist_entry(xmmsc_result_t *res, void *arg)
 {
 	struct params_entry_update *p = (struct params_entry_update *)arg;
-	static GtkTreeIter          iter;
+	GtkTreeIter                 iter;
 
 	iter = p->iter;
 
@@ -678,20 +670,57 @@ void gml_playlist_update_entry(GMedialib *gml, guint pos)
 	}
 }
 
+static void n_idlist_ready(xmmsc_result_t *res, void *pt)
+{
+	xmmsc_result_t *res2;
+	int             i, j, cnt_item = 0, length = 0, rnd[10];
+
+	for (; xmmsc_result_list_valid(res); xmmsc_result_list_next(res)) {
+		length++;
+	}
+	/* create ten random numbers between 0 and length: */
+	srand(time(NULL));
+	for (i = 0; i < 10; i++)
+		rnd[i] = rand() % length;
+	/* sort the random numbers: */
+	for (i = 0; i < 10; i++)
+		for (j = 0; j < 10; j++)
+			if (rnd[i] < rnd[j]) {
+				int b = rnd[i];
+				rnd[i] = rnd[j];
+				rnd[j] = b;
+			}
+
+	i = 0;
+	xmmsc_result_list_first(res);
+	for (; xmmsc_result_list_valid(res); xmmsc_result_list_next(res)) {
+		guint id;
+
+		if (!xmmsc_result_get_uint(res, &id)) {
+			printf("gxmms2: Broken resultset.\n");
+		}
+		if (cnt_item < 10 && i == rnd[cnt_item]) {
+			res2 = xmmsc_playlist_add_id(connection, NULL, id);
+			xmmsc_result_unref(res2);
+			cnt_item++;
+		}
+		i++;
+	}
+	xmmsc_result_unref(res);
+}
+
 static void cb_pl_add_rnd_button_pressed(GtkWidget *widget, gpointer userdata)
 {
 	xmmsc_result_t    *res;
-	gchar             *query = "SELECT id FROM Media WHERE key='url' " \
-	                           "ORDER BY RANDOM() LIMIT 10";
 	GMedialib         *gml = (GMedialib *)userdata;
-	static const char *pref[] = { "client/generic", "server", "plugins/*",
-	                              "client/*", "*", NULL };
+	xmmsc_coll_t      *universe;
 
 	gml_set_statusbar_text(gml, "Adding random songs...");
 
-	res = xmmsc_medialib_select(connection, query);
-	xmmsc_result_source_preference_set(res, pref);
-	xmmsc_result_notifier_set(res, n_query_finished, userdata);
+	universe = xmmsc_coll_universe();
+	res = xmmsc_coll_query_ids(connection, universe, NULL, 0, 0);
+
+	xmmsc_result_notifier_set(res, n_idlist_ready, userdata);
 	xmmsc_result_unref(res);
 }
 
@@ -742,7 +771,7 @@ static void cb_pl_button_delete_pressed(GtkWidget *widget, gpointer userdata)
 		xmmsc_result_t *res;
 		guint           pos = *(guint *)(element->data);
 
-		res = xmmsc_playlist_remove(connection, pos);
+		res = xmmsc_playlist_remove_entry(connection, NULL, pos);
 		xmmsc_result_notifier_set(res, n_playlist_remove, NULL);
 		xmmsc_result_unref(res);
 
@@ -819,8 +848,8 @@ static void pl_info_about_selection(GtkTreeModel  *model,
 
 	gtk_tree_model_get(model, iter, 1, &id, -1);
 	res = xmmsc_medialib_get_info(connection, id);
-    xmmsc_result_notifier_set(res, n_media_lib_get_info, NULL);
-    xmmsc_result_unref(res);
+	xmmsc_result_notifier_set(res, n_media_lib_get_info, NULL);
+	xmmsc_result_unref(res);
 }
 
 struct params
@@ -868,7 +897,7 @@ static void cb_pl_button_track_pressed(GtkWidget *widget, gpointer userdata)
 	
 	if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(p->gml->playlist.ls),
 	                                  &(p->iter))) {
-		res = xmmsc_playlist_current_pos(connection);
+		res = xmmsc_playlist_current_pos(connection, NULL);
 		xmmsc_result_notifier_set(res, n_playlist_current_pos, p);
 		xmmsc_result_unref(res);
 	}
@@ -876,38 +905,64 @@ static void cb_pl_button_track_pressed(GtkWidget *widget, gpointer userdata)
 
 static void cb_pl_save_pl_button_pressed(GtkWidget *widget, gpointer userdata);
 
-static void n_playlist_clear(xmmsc_result_t *res, void *userdata)
+static void n_playlist_loaded(xmmsc_result_t *res, void *userdata)
 {
 	GMedialib *gml = (GMedialib *)userdata;
+	gml_pl_refresh_playlist(gml);
 	gml_set_statusbar_text(gml, "Ready.");
+	xmmsc_result_unref(res);
+}
+
+static void n_playlist_clear(xmmsc_result_t *res, void *userdata)
+{
+	xmmsc_result_t *res2;
+	res2 = xmmsc_playlist_load(connection, "_gxmms2_tmp_list");
+	xmmsc_result_notifier_set(res2, n_playlist_loaded, userdata);
+	xmmsc_result_unref(res2);
+	xmmsc_result_unref(res);
 }
 
 static void cb_pl_clear_button_pressed(GtkWidget *widget, gpointer userdata)
 {
 	xmmsc_result_t *res;
 	GMedialib      *gml = (GMedialib *)userdata;
-	gint            result;
-	GtkWidget      *dialog = gtk_message_dialog_new (GTK_WINDOW(gml),
-                                  GTK_DIALOG_DESTROY_WITH_PARENT,
-                                  GTK_MESSAGE_QUESTION,
-                                  GTK_BUTTONS_NONE,
-                                  "Do you want to save the current playlist?");
-	gtk_dialog_add_buttons(GTK_DIALOG(dialog), 
-	                       GTK_STOCK_YES,    GTK_RESPONSE_YES,
-	                       GTK_STOCK_NO,     GTK_RESPONSE_NO,
-	                       GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
-	                       NULL);
+	gint            result = GTK_RESPONSE_NO;
+	GtkWidget      *checkbutton;
+	GtkWidget      *dialog;
+	
+	if (gml->playlist.new_playlist_confirm) {
+		dialog = gtk_message_dialog_new(GTK_WINDOW(gml),
+		                                GTK_DIALOG_DESTROY_WITH_PARENT,
+		                                GTK_MESSAGE_QUESTION,
+		                                GTK_BUTTONS_NONE,
+		                                "Do you want to save the current playlist?");
+		gtk_dialog_add_buttons(GTK_DIALOG(dialog), 
+		                       GTK_STOCK_YES,    GTK_RESPONSE_YES,
+		                       GTK_STOCK_NO,     GTK_RESPONSE_NO,
+		                       GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
+		                       NULL);
 
-	result = gtk_dialog_run(GTK_DIALOG(dialog));
-	if (result == GTK_RESPONSE_YES) {
-		/* save the playlist */
-		gtk_widget_destroy(dialog);
-		cb_pl_save_pl_button_pressed(widget, gml);
-	} else {
-		gtk_widget_destroy(dialog);
+		checkbutton = gtk_check_button_new_with_mnemonic("_Don't ask me again");
+		gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),
+		                  checkbutton);
+		gtk_widget_show_all(dialog);
+
+		result = gtk_dialog_run(GTK_DIALOG(dialog));
+
+		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton))) {
+			gml->playlist.new_playlist_confirm = FALSE;
+		}
+
+		if (result == GTK_RESPONSE_YES) {
+			/* save the playlist */
+			gtk_widget_destroy(dialog);
+			cb_pl_save_pl_button_pressed(widget, gml);
+		} else {
+			gtk_widget_destroy(dialog);
+		}
 	}
 	if (result != GTK_RESPONSE_REJECT) {
-		res = xmmsc_playlist_clear(connection);
+		res = xmmsc_playlist_create(connection, "_gxmms2_tmp_list");
 		xmmsc_result_notifier_set(res, n_playlist_clear, userdata);
 		xmmsc_result_unref(res);
 	}
@@ -925,6 +980,32 @@ static void n_playlist_save(xmmsc_result_t *res, void *userdata)
 	xmmsc_result_unref(res);
 }
 
+struct params_save_pl
+{
+	GMedialib *gml;
+	gchar     *pl_name_str;
+};
+
+static void n_coll_get(xmmsc_result_t *res, void *userdata)
+{
+	GMedialib *gml = ((struct params_save_pl *)userdata)->gml;;
+	if (xmmsc_result_iserror(res)) {
+		gml_set_statusbar_text(gml, (gchar *)xmmsc_result_get_error(res));
+	} else {
+		xmmsc_coll_t   *playlist_coll;
+		xmmsc_result_t *res2;
+
+		xmmsc_result_get_collection(res, &playlist_coll);
+		res2 = xmmsc_coll_save(connection, playlist_coll,
+		                       ((struct params_save_pl *)userdata)->pl_name_str,
+		                       XMMS_COLLECTION_NS_PLAYLISTS);
+		xmmsc_result_notifier_set(res2, n_playlist_save, gml);
+		xmmsc_coll_unref(playlist_coll);
+	}
+	g_free(((struct params_save_pl *)userdata)->pl_name_str);
+	xmmsc_result_unref(res);
+}
+
 static void cb_pl_save_pl_button_pressed(GtkWidget *widget, gpointer userdata)
 {
 	xmmsc_result_t *res;
@@ -940,6 +1021,7 @@ static void cb_pl_save_pl_button_pressed(GtkWidget *widget, gpointer userdata)
 	                                                     NULL);
 	GtkWidget      *label = gtk_label_new("Save as:");
 	GtkWidget      *entry = gtk_entry_new();
+	static struct   params_save_pl p;
 
 	gtk_widget_set_size_request(GTK_WIDGET(dialog), 300, 100);
 	gtk_window_set_resizable(GTK_WINDOW(dialog), FALSE);
@@ -949,16 +1031,109 @@ static void cb_pl_save_pl_button_pressed(GtkWidget *widget, gpointer userdata)
 
 	result = gtk_dialog_run(GTK_DIALOG(dialog));
 	if (result == GTK_RESPONSE_ACCEPT) {
+		gint   len = strlen(gtk_entry_get_text(GTK_ENTRY(entry)));
+		gchar *pl_name_str = g_malloc(len);
+		strncpy(pl_name_str, gtk_entry_get_text(GTK_ENTRY(entry)), len);
 		/* save the playlist */
-		res = xmmsc_medialib_playlist_save_current(connection, 
-		                                           gtk_entry_get_text(
-		                                           GTK_ENTRY(entry)));
-		xmmsc_result_notifier_set(res, n_playlist_save, userdata);
+		res = xmmsc_coll_get(connection, "_active", "Playlists");
+		p.gml = (GMedialib *)userdata;
+		p.pl_name_str = pl_name_str;
+		xmmsc_result_notifier_set(res, n_coll_get, &p);
 		xmmsc_result_unref(res);
 	}
 	gtk_widget_destroy(dialog);
 }
 
+static void cb_pl_drag_data_get(GtkWidget        *widget,
+                                GdkDragContext   *drag_context,
+                                GtkSelectionData *selection,
+                                guint             info,
+                                guint             time,
+                                gpointer          pt)
+{
+	GtkTreeSelection *tree_sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
+	GtkTreeModel     *model = gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
+	GList            *sel = gtk_tree_selection_get_selected_rows(tree_sel, &model);
+	gchar            *rows = NULL;
+
+	for (; sel; sel = g_list_next(sel)) {
+		gchar *path_string = gtk_tree_path_to_string((GtkTreePath*) sel->data);
+
+		if (rows == NULL) {
+			rows = path_string;
+		} else {
+			gchar *str = g_strconcat(rows, ";", path_string, NULL);
+			g_free(rows);
+			g_free(path_string);
+			rows = str;
+		}
+	}
+	g_list_free(sel);
+
+	gtk_selection_data_set(selection, drag_atom, sizeof(guchar), 
+	                       (guchar *)rows, strlen(rows) + 1);
+}
+
+static void cb_pl_drag_data_received(GtkWidget        *widget,
+                                     GdkDragContext   *drag_context,
+                                     gint              x,
+                                     gint              y,
+                                     GtkSelectionData *selection,
+                                     guint             info,
+                                     guint             time,
+                                     gpointer          userdata)
+{
+	guchar                   *data_text = selection->data;
+	gchar                   **rows = g_strsplit((gchar *)data_text, ";", -1);
+	gchar                   **cur_row;
+	GtkTreePath              *path;
+	GtkTreeViewDropPosition   position;
+	GtkTreeModel             *model;
+	GtkTreeIter               iter;
+	guint                     pos;
+	guint                     i;
+	GMedialib                *gml = (GMedialib *)userdata;
+
+	g_return_if_fail(data_text != NULL);
+
+	if (!gtk_tree_view_get_dest_row_at_pos(
+		GTK_TREE_VIEW(widget), x, y, &path, &position))
+		return;
+
+	model = gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
+	gtk_tree_model_get_iter(model, &iter, path);
+	gtk_tree_model_get(model, &iter, 0, &pos, -1);
+
+	for (cur_row = rows; *cur_row; cur_row++) {
+		GtkTreeIter     old_iter;
+		guint           old_pos;
+		xmmsc_result_t *res = NULL;
+
+		gtk_tree_model_get_iter_from_string(model, &old_iter, *cur_row);
+		gtk_tree_model_get(model, &old_iter, 0, &old_pos, -1);
+
+		if ((gint)pos - (gint)old_pos < 0) {
+			res = xmmsc_playlist_move_entry(connection, NULL, old_pos, pos);
+			gtk_list_store_move_before(GTK_LIST_STORE(model),
+			                           &old_iter, &iter);
+		} else {
+			res = xmmsc_playlist_move_entry(connection, NULL, old_pos, pos-1);
+			gtk_list_store_move_before(GTK_LIST_STORE(model),
+			                           &old_iter, &iter);
+		}
+		xmmsc_result_unref(res);
+	}
+	g_strfreev(rows);
+
+	gtk_tree_model_get_iter_first(model, &iter);
+	i = 0;
+	do {
+		gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, i, -1);
+		i++;
+	} while (gtk_tree_model_iter_next(model, &iter));
+	update_current_pl_pos(gml);
+}
+
 static void cb_pl_reload_button_pressed(GtkWidget *widget, gpointer pt)
 {
 	GMedialib *gml = (GMedialib *)pt;
@@ -1220,11 +1395,11 @@ static void pl_view_popup_menu(GtkWidget      *treeview,
 	};
 
 	struct submenu sm_columns[] = 
-		{ {"#", gml_pl_get_column_pos_visible, cb_column_pos_visible},
-		  {"ID", gml_pl_get_column_id_visible, cb_column_id_visible},
+		{ {"#",      gml_pl_get_column_pos_visible,    cb_column_pos_visible},
+		  {"ID",     gml_pl_get_column_id_visible,     cb_column_id_visible},
 		  {"Artist", gml_pl_get_column_artist_visible, cb_column_artist_visible},
-		  {"Track", gml_pl_get_column_track_visible, cb_column_track_visible},
-		  {"Album", gml_pl_get_column_album_visible, cb_column_album_visible},
+		  {"Track",  gml_pl_get_column_track_visible,  cb_column_track_visible},
+		  {"Album",  gml_pl_get_column_album_visible,  cb_column_album_visible},
 		  {"Rating", gml_pl_get_column_rating_visible, cb_column_rating_visible},
 		  { NULL, NULL, NULL } };
 
@@ -1390,6 +1565,16 @@ gboolean gml_pl_get_column_rating_visible(GMedialib *gml)
 	return gml->playlist.column_rating_visible;
 }
 
+gboolean gml_pl_get_new_playlist_confirm(GMedialib *gml)
+{
+	return gml->playlist.new_playlist_confirm;
+}
+
+void gml_pl_set_new_playlist_confirm(GMedialib *gml, gboolean confirm)
+{
+	gml->playlist.new_playlist_confirm = confirm;
+}
+
 void gml_pl_set_column_pos_visible(GMedialib *gml, gboolean visible)
 {
 	gml->playlist.column_pos_visible = visible;
@@ -1442,12 +1627,14 @@ void gml_create_playlist(GMedialib *gml)
 	GtkTreeSelection *selection;
 	GtkTooltips      *tooltips;
 	GdkPixbuf        *icon;
+	GtkWidget        *album_item, *artist_item, *title_item, *tracknr_item;
+	GtkTargetEntry    row_targets[1];
 
 	gml->playlist.frame = gtk_frame_new("Playlist");
 	gtk_container_set_border_width(GTK_CONTAINER(gml->playlist.frame), 2);
 
 	/* table playlist */
-	gml->playlist.table = gtk_table_new(2, 9, FALSE);
+	gml->playlist.table = gtk_table_new(2, 7, FALSE);
 	gtk_container_set_border_width(GTK_CONTAINER(gml->playlist.table), 3);
 	gtk_table_set_row_spacings(GTK_TABLE(gml->playlist.table), 4);
 	gtk_table_set_col_spacings(GTK_TABLE(gml->playlist.table), 4);
@@ -1515,6 +1702,33 @@ void gml_create_playlist(GMedialib *gml)
 	gtk_table_attach(GTK_TABLE(gml->playlist.table),
 		             gml->playlist.button_sort_pl,
 		             4, 5, 0, 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
+	gml->playlist.menu_sort_pl = gtk_menu_new();
+
+	/* sort playlist menu */
+	album_item   = gtk_menu_item_new_with_label("Album");
+	artist_item  = gtk_menu_item_new_with_label("Artist");
+	title_item   = gtk_menu_item_new_with_label("Title");
+	tracknr_item = gtk_menu_item_new_with_label("Tracknr");
+	gtk_menu_shell_append(GTK_MENU_SHELL(gml->playlist.menu_sort_pl), album_item);
+	gtk_menu_shell_append(GTK_MENU_SHELL(gml->playlist.menu_sort_pl), artist_item);
+	gtk_menu_shell_append(GTK_MENU_SHELL(gml->playlist.menu_sort_pl), title_item);
+	gtk_menu_shell_append(GTK_MENU_SHELL(gml->playlist.menu_sort_pl), tracknr_item);
+	gtk_widget_show(album_item);
+	gtk_widget_show(artist_item);
+	gtk_widget_show(title_item);
+	gtk_widget_show(tracknr_item);
+	gtk_signal_connect(GTK_OBJECT(album_item),   "activate",
+	                   G_CALLBACK(cb_pl_sort_menu_activate),
+					   (gpointer)gml);
+	gtk_signal_connect(GTK_OBJECT(artist_item),  "activate",
+                       G_CALLBACK(cb_pl_sort_menu_activate),
+					   (gpointer)gml);
+	gtk_signal_connect(GTK_OBJECT(title_item),   "activate",
+	                   G_CALLBACK(cb_pl_sort_menu_activate),
+					   (gpointer)gml);
+	gtk_signal_connect(GTK_OBJECT(tracknr_item), "activate",
+	                   G_CALLBACK(cb_pl_sort_menu_activate),
+					   (gpointer)gml);
 
 	/* reload playlist button */
 	gml->playlist.button_reload_pl = gtk_button_new();
@@ -1541,31 +1755,6 @@ void gml_create_playlist(GMedialib *gml)
 		             6, 7, 0, 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
 	gtk_widget_set_sensitive(gml->playlist.button_track, TRUE);
 
-	/* up button */
-	gml->playlist.button_up = gtk_button_new();
-	gml->playlist.image_up = gtk_image_new_from_stock(GTK_STOCK_GO_UP, 
-	                                      GTK_ICON_SIZE_LARGE_TOOLBAR);
-	gtk_button_set_relief(GTK_BUTTON(gml->playlist.button_up), GTK_RELIEF_NONE);
-	gtk_container_add(GTK_CONTAINER(gml->playlist.button_up),
-	                  gml->playlist.image_up);
-	gtk_table_attach(GTK_TABLE(gml->playlist.table),
-		             gml->playlist.button_up,
-		             7, 8, 0, 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
-	gtk_widget_set_sensitive(gml->playlist.button_up, FALSE);
-
-	/* down button */
-	gml->playlist.button_down = gtk_button_new();
-	gml->playlist.image_down = gtk_image_new_from_stock(GTK_STOCK_GO_DOWN, 
-	                                          GTK_ICON_SIZE_LARGE_TOOLBAR);
-	gtk_button_set_relief(GTK_BUTTON(gml->playlist.button_down),
-	                      GTK_RELIEF_NONE);
-	gtk_container_add(GTK_CONTAINER(gml->playlist.button_down),
-	                  gml->playlist.image_down);
-	gtk_table_attach(GTK_TABLE(gml->playlist.table),
-		             gml->playlist.button_down,
-		             8, 9, 0, 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
-	gtk_widget_set_sensitive(gml->playlist.button_down, FALSE);
-
 	/* list playlist */
 	gml->playlist.ls = gtk_list_store_new(6, G_TYPE_INT, G_TYPE_INT, 
 	                                      G_TYPE_STRING, G_TYPE_STRING,
@@ -1575,6 +1764,22 @@ void gml_create_playlist(GMedialib *gml)
 		gtk_tree_view_new_with_model(GTK_TREE_MODEL(gml->playlist.ls));
 	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(gml->playlist.list), TRUE);
 
+	row_targets[0].target = "PLAYLIST_MODEL_ROW";
+	row_targets[0].flags = GTK_TARGET_SAME_WIDGET;
+	row_targets[0].info = 0;
+	drag_atom = gdk_atom_intern(row_targets[0].target, FALSE);
+	gtk_tree_view_enable_model_drag_source(
+		GTK_TREE_VIEW(gml->playlist.list),
+		GDK_BUTTON1_MASK,
+		row_targets,
+		G_N_ELEMENTS(row_targets),
+		GDK_ACTION_MOVE);
+	gtk_tree_view_enable_model_drag_dest(
+		GTK_TREE_VIEW(gml->playlist.list),
+		row_targets,
+		G_N_ELEMENTS(row_targets),
+		GDK_ACTION_MOVE);
+
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(gml->playlist.list));
     gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
 
@@ -1588,7 +1793,7 @@ void gml_create_playlist(GMedialib *gml)
 
 	gtk_table_attach(GTK_TABLE(gml->playlist.table),
 		             gml->playlist.scroll_widget,
-		             0, 9, 1, 2,
+		             0, 7, 1, 2,
 		             GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 0, 0);
 
 	gml->playlist.column_pos = gtk_tree_view_column_new_with_attributes("#", 
@@ -1675,16 +1880,12 @@ void gml_create_playlist(GMedialib *gml)
 	gtk_tooltips_set_tip(tooltips, gml->playlist.button_shuffle_pl, 
 	                     "Shuffle the playlist", NULL);
 	gtk_tooltips_set_tip(tooltips, gml->playlist.button_sort_pl, 
-	                     "Sort the playlist by artist", NULL);
+	                     "Sort the playlist", NULL);
 	gtk_tooltips_set_tip(tooltips, gml->playlist.button_add_rnd, 
 	                     "Add 10 random songs to the playlist", NULL);
 	gtk_tooltips_set_tip(tooltips, gml->playlist.button_reload_pl, 
 	                     "Refresh the playlist\n(Useful if another XMMS2 " \
 	                     "client has altered the playlist)", NULL);
-	gtk_tooltips_set_tip(tooltips, gml->playlist.button_up, 
-	                     "Move selected track one position upwards", NULL);
-	gtk_tooltips_set_tip(tooltips, gml->playlist.button_down, 
-	                     "Move selected track one position downwards", NULL);
 	gtk_tooltips_set_tip(tooltips, gml->playlist.button_track, 
 	                     "Show currently played track", NULL);
 
@@ -1695,15 +1896,21 @@ void gml_create_playlist(GMedialib *gml)
 	gml_pl_set_column_album_visible (gml, TRUE);
 	gml_pl_set_column_rating_visible(gml, TRUE);
 
+	g_signal_connect(G_OBJECT(gml->playlist.list),              "drag-data-get",
+	                 G_CALLBACK(cb_pl_drag_data_get),
+	                 (gpointer)gml);
+	g_signal_connect(G_OBJECT(gml->playlist.list),              "drag_data_received",
+	                 G_CALLBACK(cb_pl_drag_data_received),
+	                 (gpointer)gml);
 	g_signal_connect(G_OBJECT(gml->playlist.button_reload_pl),  "clicked",
 	                 G_CALLBACK(cb_pl_reload_button_pressed),
 	                 (gpointer)gml);
 	g_signal_connect(G_OBJECT(gml->playlist.button_shuffle_pl), "clicked",
 	                 G_CALLBACK(cb_pl_shuffle_button_pressed),
 	                 (gpointer)gml);
-	g_signal_connect(G_OBJECT(gml->playlist.button_sort_pl),    "clicked",
-	                 G_CALLBACK(cb_pl_sort_button_pressed),
-	                 (gpointer)gml);
+	g_signal_connect(G_OBJECT(gml->playlist.button_sort_pl),    "button-press-event",
+					 G_CALLBACK(cb_pl_sort_button_raw_pressed),
+					 (gpointer)gml);
 	g_signal_connect(G_OBJECT(gml->playlist.button_add_rnd),    "clicked",
 	                 G_CALLBACK(cb_pl_add_rnd_button_pressed),
 	                 (gpointer)gml);
@@ -1716,27 +1923,18 @@ void gml_create_playlist(GMedialib *gml)
 	g_signal_connect(G_OBJECT(gml->playlist.button_track),      "clicked",
 	                 G_CALLBACK(cb_pl_button_track_pressed),
 	                 (gpointer)gml);
-	g_signal_connect(G_OBJECT(gml->playlist.button_up),         "clicked",
-	                 G_CALLBACK(cb_pl_button_up_pressed),
-	                 (gpointer)gml);
-	g_signal_connect(G_OBJECT(gml->playlist.button_down),       "clicked",
-	                 G_CALLBACK(cb_pl_button_down_pressed),
-	                 (gpointer)gml);
-	g_signal_connect(G_OBJECT(gml->playlist.list),        "row-activated",
+	g_signal_connect(G_OBJECT(gml->playlist.list),              "row-activated",
 	                 G_CALLBACK(cb_pl_on_tree_view_row_activated),
 	                 (gpointer)gml);
 	gml->playlist.sel = 
 		gtk_tree_view_get_selection(GTK_TREE_VIEW(gml->playlist.list));
-	g_signal_connect(gml->playlist.sel,                         "changed",
-	                 G_CALLBACK(cb_pl_on_tree_selection_changed),
-	                 (gpointer)gml);
-	g_signal_connect(G_OBJECT(gml->playlist.list),   "button-press-event",
+	g_signal_connect(G_OBJECT(gml->playlist.list),              "button-press-event",
 	                 G_CALLBACK(cb_pl_selection_button_press),
 	                 (gpointer)gml);
-	g_signal_connect(G_OBJECT(gml->playlist.list),           "popup-menu",
+	g_signal_connect(G_OBJECT(gml->playlist.list),              "popup-menu",
 	                 G_CALLBACK(cb_pl_selection_popup_menu),
 	                 (gpointer)gml);
-	g_signal_connect(G_OBJECT(gml->playlist.list),      "key-press-event",
+	g_signal_connect(G_OBJECT(gml->playlist.list),              "key-press-event",
 	                 G_CALLBACK(cb_pl_selection_key_press),
 	                 (gpointer)gml);
 	gml_pl_setup_xmms_callbacks(gml);
@@ -1745,4 +1943,5 @@ void gml_create_playlist(GMedialib *gml)
 	gml->playlist.move_finished = TRUE;
 	gml->playlist.refresh = FALSE;
 	gml->playlist.refresh_counter = 0;
+	gml->playlist.new_playlist_confirm = TRUE;
 }
