@@ -2,7 +2,7 @@
  * File: gmlsearch.c
  *
  * Medialib search and playlist editor
- * Copyright (c) 2005-2006 Johannes Heimansberg
+ * Copyright (c) 2005-2007 Johannes Heimansberg
  *
  * requires GTK+ 2.6 or better and XMMS2
  *
@@ -19,6 +19,8 @@
 #include "gtrackinfo/gtrackinfo.h"
 #include "sdecode/sdecode.h"
 
+void gmedialib_refresh(GMedialib*, xmmsc_connection_t*);
+
 extern xmmsc_connection_t *connection; /* available from gxmms2.c */
 
 struct search_params
@@ -28,6 +30,13 @@ struct search_params
 	gchar              *category;
 	gchar              *search_str;
 	gboolean            start;
+	xmmsc_coll_t       *coll_query;
+};
+
+struct rehash_notify_data
+{
+	GMedialib *gml;
+	guint      remaining_ids;
 };
 
 void gml_search_do_search(GMedialib *gml, guint search_for, gchar* search_str)
@@ -56,7 +65,7 @@ static void cb_search_on_tree_view_row_activated(GtkTreeView       *view,
 	if (gtk_tree_model_get_iter(model, &iter, path)) {
 		gtk_tree_model_get(model, &iter, 0, &id, -1);
 
-		res = xmmsc_playlist_add_id(connection, id);
+		res = xmmsc_playlist_add_id(connection, NULL, id);
 		xmmsc_result_unref(res);
 	}
 }
@@ -71,7 +80,7 @@ static void add_selected_foreach_func(GtkTreeModel *model,
 
 	gtk_tree_model_get(model, selected_row, 0, &id, -1);
 
-	res = xmmsc_playlist_add_id(connection, id);
+	res = xmmsc_playlist_add_id(connection, NULL, id);
 	xmmsc_result_unref(res);
 }
 
@@ -97,7 +106,7 @@ static void insert_selected_foreach_func(GtkTreeModel *model,
 
 	gtk_tree_model_get(model, selected_row, 0, &id, -1);
 
-	res = xmmsc_playlist_insert_id(connection, pos + 1, id);
+	res = xmmsc_playlist_insert_id(connection, NULL, pos + 1, id);
 	xmmsc_result_unref(res);
 }
 
@@ -116,29 +125,106 @@ static void cb_search_popup_insert_pressed(GtkWidget *widget, gpointer pt)
 
 }
 
+static void n_rehash(xmmsc_result_t *res, void *userdata)
+{   
+	struct rehash_notify_data* data = (struct rehash_notify_data *)userdata;
+
+	data->remaining_ids--;
+	
+	if (data->remaining_ids == 0)
+		gmedialib_refresh(data->gml, connection);
+}   
+
+static void cb_search_rehash_button_pressed(GtkWidget *widget, gpointer pt)
+{
+	GMedialib        *gml = (GMedialib *)pt;
+	GtkTreeView      *tree_view = GTK_TREE_VIEW(gml->search.list_results);
+	GtkTreeSelection *selection = gtk_tree_view_get_selection(tree_view);
+	GtkTreeModel     *model = gtk_tree_view_get_model(tree_view);
+	GList            *sel, *cur;
+	static struct rehash_notify_data data;
+
+	sel = gtk_tree_selection_get_selected_rows(selection, &model);
+	data.gml = gml;
+	data.remaining_ids = g_list_length(sel);
+	
+	for (cur = sel; cur; cur = g_list_next(cur)) {
+		GtkTreeIter     iter;
+		xmmsc_result_t *res;
+		guint           id;
+
+		gtk_tree_model_get_iter(model, &iter, (GtkTreePath*)cur->data);
+		gtk_tree_model_get(model, &iter, 0, &id, -1);
+
+		res = xmmsc_medialib_rehash (connection, id);
+		xmmsc_result_notifier_set(res, n_rehash, &data);
+		xmmsc_result_unref(res);
+	}
+	g_list_free(sel);
+}
+
+static void cb_search_remove_button_pressed(GtkWidget *widget, gpointer pt)
+{
+	GMedialib        *gml = (GMedialib *)pt;
+	GtkTreeView      *tree_view = GTK_TREE_VIEW(gml->search.list_results);
+	GtkTreeSelection *selection = gtk_tree_view_get_selection(tree_view);
+	GtkTreeModel     *model = gtk_tree_view_get_model(tree_view);
+	GList            *sel, *cur;
+
+	sel = g_list_last(gtk_tree_selection_get_selected_rows(selection, &model));
+	for (cur = sel; cur; cur = g_list_previous(cur)) {
+		GtkTreeIter     iter;
+		xmmsc_result_t *res;
+		guint           id;
+
+		gtk_tree_model_get_iter(model, &iter, (GtkTreePath*) cur->data);
+		gtk_tree_model_get(model, &iter, 0, &id, -1);
+
+		res = xmmsc_medialib_remove_entry(connection, id);
+		xmmsc_result_unref(res);
+
+		gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
+	}
+	g_list_free(sel);
+}
+
+gchar* map_category(gint category)
+{
+	gchar *str = NULL;
+
+	switch (category) {
+		case 0:
+			break;
+		case 1:
+			str = "artist";
+			break;
+		case 2:
+			str = "title";
+			break;
+		case 3:
+			str = "album";
+			break;
+		case 4:
+			str = "url";
+			break;
+		default:
+			g_return_val_if_reached(NULL);
+	}
+
+	return g_strdup(str);
+}
+
 static void cb_search_search_button_pressed(GtkWidget *widget, gpointer pt)
 {
 	GMedialib *gml = (GMedialib *)pt;
-	gchar     *str, *category = NULL;
+	gchar     *str, *category;
+	gint       active_category;
 	gboolean   exact;
 
 	str = (gchar *)gtk_entry_get_text(GTK_ENTRY(gml->search.entry_searchstring));
-
-	switch (gtk_combo_box_get_active(GTK_COMBO_BOX(gml->search.search_combo))) {
-		case 0:
-			category = NULL;
-			break;
-		case 1:
-			category = "artist";
-			break;
-		case 2:
-			category = "title";
-			break;
-		case 3:
-			category = "album";
-			break;
-	}
-
+	active_category = gtk_combo_box_get_active(
+		GTK_COMBO_BOX(gml->search.search_combo));
+	category = map_category(active_category);	
 	exact = gtk_toggle_button_get_active(
 	        	GTK_TOGGLE_BUTTON(gml->search.checkbutton_exact));
 
@@ -149,7 +235,14 @@ static void cb_search_search_button_pressed(GtkWidget *widget, gpointer pt)
 		gtk_widget_set_sensitive(gml->search.checkbutton_exact,  FALSE);
 		if (gml->search.ls != NULL)
 			gtk_list_store_clear(gml->search.ls);
+
+		g_free(gml->search.last_search_str);
+		gml->search.last_search_str = g_strdup(str);
+		gml->search.last_category = active_category;
+		gml->search.last_exact = exact;
+
 		gmedialib_search(gml, connection, category, str, exact);
+		g_free(category);
 	}
 }
 
@@ -377,6 +470,29 @@ static void search_view_popup_menu(GtkWidget      *treeview,
 
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
+	menuitem =  gtk_image_menu_item_new_with_mnemonic("Re_hash");
+	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem),
+	                              gtk_image_new_from_stock(GTK_STOCK_EXECUTE,
+								                           GTK_ICON_SIZE_SMALL_TOOLBAR));
+	
+	if (none) gtk_widget_set_sensitive(menuitem, FALSE);
+
+	g_signal_connect(menuitem, "activate",
+	                 G_CALLBACK(cb_search_rehash_button_pressed), userdata);
+
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+
+	menuitem =  gtk_image_menu_item_new_with_mnemonic("_Remove from Medialib");
+	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem),
+	                              gtk_image_new_from_stock(GTK_STOCK_DELETE,
+								                           GTK_ICON_SIZE_SMALL_TOOLBAR));
+	if (none) gtk_widget_set_sensitive(menuitem, FALSE);
+
+	g_signal_connect(menuitem, "activate",
+                     G_CALLBACK(cb_search_remove_button_pressed), userdata);
+	
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+
 	menuitem = gtk_separator_menu_item_new();
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
@@ -421,23 +537,23 @@ static void n_add_result_notified(xmmsc_result_t *res, void *arguments)
 	static gint           tmp_id = -1;
 	GtkTreeIter           iter;
 
-	xmmsc_result_get_dict_entry_int32(res, "id",      &id);
-	xmmsc_result_get_dict_entry_int32(res, "tracknr", &tracknr);
-	xmmsc_result_get_dict_entry_str(  res, "title",   &title);
-	xmmsc_result_get_dict_entry_int32(res, "rating",  &rating);
+	xmmsc_result_get_dict_entry_int(res,    "id",      &id);
+	xmmsc_result_get_dict_entry_int(res,    "tracknr", &tracknr);
+	xmmsc_result_get_dict_entry_string(res, "title",   &title);
+	xmmsc_result_get_dict_entry_int(res,    "rating",  &rating);
 	rating--;
 	if (title) {
-		xmmsc_result_get_dict_entry_str(res, "artist", &artist);
+		xmmsc_result_get_dict_entry_string(res, "artist", &artist);
 		if (!artist)
 			artist = "[Unknown Artist]";
 
-		xmmsc_result_get_dict_entry_str(res, "album", &album);
+		xmmsc_result_get_dict_entry_string(res, "album", &album);
 		if (!album)
 			album = "[Unknown Album]";
 	} else {
 		gchar       *url, *url_utf8;
 		const gchar *url_tmp;
-		xmmsc_result_get_dict_entry_str(res, "url", &url);
+		xmmsc_result_get_dict_entry_string(res, "url", &url);
 		if (url) {
 			url_tmp = xmmsc_result_decode_url(res, url);
 			url_utf8 = get_valid_utf8_str_alloc(url_tmp);
@@ -508,16 +624,17 @@ static void n_continue_search(xmmsc_result_t *res, void *arguments)
 	}
 
 	for (; xmmsc_result_list_valid(res); xmmsc_result_list_next(res)) {
-		gint32 *id = g_malloc(sizeof(gint32));
+		guint32 *id = g_malloc(sizeof(guint32));
 
-		xmmsc_result_get_dict_entry_int32(res, "id", id);
-		if (!id)
-			printf("broken resultset\n");
+		if (!xmmsc_result_get_uint(res, id)) {
+			printf("gxmms2: Broken resultset.\n");
+		}
 
 		n = g_list_prepend(n, id);
 	}
 
 	add_results(n, arg);
+	xmmsc_coll_unref(arg->coll_query);
 	xmmsc_result_unref(res);
 
 	gml_set_statusbar_text(arg->gml, "Ready.");
@@ -605,48 +722,50 @@ gchar *mask_sql_quotes(gchar *string)
 void gmedialib_search(GMedialib *gml,  xmmsc_connection_t *conn, 
                       gchar *category, gchar *search_str, gboolean exact)
 {
-	gchar                        query[1024];
-	gchar                       *search_str_masked;
 	xmmsc_result_t              *res;
 	static struct search_params  p;
+	xmmsc_coll_t                 *coll_query;
+	gchar                         pattern[1024];
 
-	search_str_masked = mask_sql_quotes(search_str);
+	if (category == NULL) category = "";
 
-	if (category != NULL && exact == FALSE)
-		g_snprintf(query, sizeof(query), 
-	               "SELECT id FROM Media WHERE LOWER(key) = LOWER('%s') " \
-	               "and LOWER(value) LIKE LOWER('%%%s%%') ORDER BY 1 DESC",
-	               category, search_str_masked);
-	else if (category != NULL && exact == TRUE)
-		g_snprintf(query, sizeof(query), 
-	               "SELECT id FROM Media WHERE LOWER(key) = LOWER('%s') " \
-	               "and LOWER(value) LIKE LOWER('%s') ORDER BY 1 DESC",
-	               category, search_str_masked);
-	else if (category == NULL && exact == FALSE)
-		g_snprintf(query, sizeof(query), 
-	               "SELECT id FROM Media WHERE LOWER(value) LIKE " \
-	               "LOWER('%%%s%%') ORDER BY 1 DESC",
-	               search_str_masked);
-	else if (category == NULL && exact == TRUE)
-		g_snprintf(query, sizeof(query), 
-	               "SELECT id FROM Media WHERE LOWER(value) LIKE " \
-	               "LOWER('%s') ORDER BY 1 DESC",
-	               search_str_masked);
+	gml_set_statusbar_text(gml, "Searching...");
+	if (!exact)
+		snprintf(pattern, 1023, "%s~\"%s\"", category, search_str);
+	else
+		snprintf(pattern, 1023, "%s:\"%s\"", category, search_str);
 
-	g_free(search_str_masked);
+	if (!xmmsc_coll_parse(pattern, &coll_query))
+		printf("gxmms2: Unable to generate query: %s\n", pattern);
+	res = xmmsc_coll_query_ids(conn, coll_query, NULL, 0, 0);
 
 	p.gml = gml;
 	p.conn = connection;
 	p.category = category;
 	p.search_str = search_str;
+	p.coll_query = coll_query;
 
-	gml_set_statusbar_text(gml, "Searching...");
-
-	res = xmmsc_medialib_select(conn, query);
 	xmmsc_result_notifier_set(res, n_continue_search, &p);
 	xmmsc_result_unref(res);
 }
 
+void gmedialib_refresh(GMedialib *gml, xmmsc_connection_t *conn)
+{
+	gchar *category = map_category(gml->search.last_category);
+
+	gtk_entry_set_text(
+		GTK_ENTRY(gml->search.entry_searchstring), gml->search.last_search_str);
+	gtk_combo_box_set_active(
+		GTK_COMBO_BOX(gml->search.search_combo), gml->search.last_category);
+	gtk_toggle_button_set_active(
+		GTK_TOGGLE_BUTTON(gml->search.checkbutton_exact), gml->search.last_exact);
+	
+	gmedialib_search(gml, conn,	category, gml->search.last_search_str,
+	                 gml->search.last_exact);
+	
+	g_free(category);
+}
+
 void gml_create_mlib_search(GMedialib *gml)
 {
 	GtkCellRenderer  *renderer_pixbuf;
@@ -680,6 +799,9 @@ void gml_create_mlib_search(GMedialib *gml)
 	                          "Track");
 	gtk_combo_box_append_text(GTK_COMBO_BOX(gml->search.search_combo), 
 	                          "Album");
+	gtk_combo_box_append_text(GTK_COMBO_BOX(gml->search.search_combo), 
+	                          "URL");
+
 
 	gtk_combo_box_set_active(GTK_COMBO_BOX(gml->search.search_combo), 0);
 
