@@ -2,16 +2,16 @@
  * File: gxmms2.c
  *
  * gxmms2 - A gtk2 frontend for XMMS2
- * Copyright (c) 2005-2006 Johannes Heimansberg
+ * Copyright (c) 2005-2007 Johannes Heimansberg
  *
  * requires GTK+ 2.6 or better and XMMS2
  *
- * Version: 0.6.4
+ * Version: 0.7.0
  *
  * Released under the GNU General Public License v2
  */
 
-#define VERSION_NUMBER "0.6.4"
+#define VERSION_NUMBER "0.7.0"
 
 #include <stdlib.h>
 #include <string.h>
@@ -113,7 +113,7 @@ static void bc_handle_volume_change(xmmsc_result_t *res, void *userdata)
 	guint32 volume;
 	gchar   buf[64];
 	
-	xmmsc_result_get_dict_entry_uint32(res, "left", &volume);
+	xmmsc_result_get_dict_entry_uint(res, "left", &volume);
 	wejpvolume_set_volume(WEJPVOLUME(wv), volume);
 	snprintf(buf, 63, "Volume: %d %%", volume);
 	wejpscroller_shortmessage_show(WEJPSCROLLER(scroller), buf, 50);
@@ -123,7 +123,7 @@ static void n_volume_init(xmmsc_result_t *res, void *userdata)
 {
 	guint32 volume;
 	
-	xmmsc_result_get_dict_entry_uint32(res, "left", &volume);
+	xmmsc_result_get_dict_entry_uint(res, "left", &volume);
 	wejpvolume_set_volume(WEJPVOLUME(wv), volume);
 	change_volume(0, volume);
 	xmmsc_result_unref(res);
@@ -142,7 +142,7 @@ static void show_gxmms2_about_dialog(void)
 	gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(dialog), 
 	                             VERSION_NUMBER);
 	gtk_about_dialog_set_copyright(GTK_ABOUT_DIALOG(dialog),
-	                               "Copyright (c) 2005-2006 " \
+	                               "Copyright (c) 2005-2007 " \
 	                               "Johannes Heimansberg");
 	gtk_about_dialog_set_comments(GTK_ABOUT_DIALOG(dialog),
 	                               "A XMMS2 frontend written in C " \
@@ -150,6 +150,9 @@ static void show_gxmms2_about_dialog(void)
 	                               "the GNU General Public License v2.");
 	gtk_about_dialog_set_website(GTK_ABOUT_DIALOG(dialog),
 	                             "http://wejp.k.vu/");
+	g_signal_connect(G_OBJECT(dialog), "response",
+	                 G_CALLBACK(gtk_widget_destroy),
+	                 (gpointer)dialog);
 	gtk_dialog_run(GTK_DIALOG(dialog));
 }
 
@@ -166,6 +169,10 @@ static void load_medialib_config(ConfigFile *cf, GMedialib *playlist)
 {
 		gml_set_font_size(GMEDIALIB(playlist),
 		                  atoi(cfg_get_key_value(*cf, "MediaLibFontSizePt")));
+		if (strncasecmp(cfg_get_key_value(*cf, "MediaLibPlaylistNewConfirm"), 
+		                "no", 3) == 0) {
+			gml_pl_set_new_playlist_confirm(GMEDIALIB(playlist), FALSE);
+		}		
 		if (strncasecmp(cfg_get_key_value(*cf, "RestoreMediaLibSize"), 
 		                "yes", 3) == 0) {
 			gtk_window_resize(GTK_WINDOW(playlist),
@@ -655,7 +662,7 @@ static void cb_drag_data_received(GtkWidget          *widget,
 				m = 0;
 				/* decode %XX escape sequences to their corresponding chars: */
 				uri_conv = decode_string(buffer);
-				res = xmmsc_playlist_add(connection, uri_conv);
+				res = xmmsc_playlist_add_url(connection, NULL, uri_conv);
 				xmmsc_result_unref(res);
 				g_free(uri_conv);
 			} else if (((guchar *)data->data)[n] > 31) {
@@ -681,6 +688,7 @@ static void load_config(gchar *path_to_config_file)
 	cfg_add_key(&cf, "ScrollerFont",                  "Sans Serif");
 	cfg_add_key(&cf, "ScrollerFontSizePx",            "12");
 	cfg_add_key(&cf, "ScrollerDelay",                 "27");
+	cfg_add_key(&cf, "SmartScroller",                 "yes");
 	cfg_add_key(&cf, "MainWindowPosX",                "10");
 	cfg_add_key(&cf, "MainWindowPosY",                "10");
 	cfg_add_key(&cf, "MediaLibWindowPosX",            "60");
@@ -703,6 +711,7 @@ static void load_config(gchar *path_to_config_file)
 	cfg_add_key(&cf, "MediaLibPlaylistColumnTrack",   "show");
 	cfg_add_key(&cf, "MediaLibPlaylistColumnAlbum",   "show");
 	cfg_add_key(&cf, "MediaLibPlaylistColumnRating",  "show");
+	cfg_add_key(&cf, "MediaLibPlaylistNewConfirm",    "yes");
 	cfg_add_key(&cf, "MediaLibAlbumTrackNrDigits",    "3");
 	cfg_add_key(&cf, "TrayIcon",                      "yes");
 	cfg_add_key(&cf, "AutoReconnect",                 "no");
@@ -924,7 +933,9 @@ int main(int argc, char *argv[])
 		gtk_widget_set_size_request(GTK_WIDGET(hbox), -1, scroller_font_size + 3);
 
 	/* scroller */
-	scroller = wejpscroller_new(atoi(cfg_get_key_value(cf, "ScrollerDelay")));
+	scroller = wejpscroller_new(atoi(cfg_get_key_value(cf, "ScrollerDelay")),
+	                            strncmp(cfg_get_key_value(cf, "SmartScroller"),
+	                                    "yes", 3) == 0 ? 1 : 0);
 	wejpscroller_set_dimensions(WEJPSCROLLER(scroller), 120, 
 	                            scroller_font_size + 2);
 	wejpscroller_set_font(WEJPSCROLLER(scroller),
@@ -1124,6 +1135,12 @@ int main(int argc, char *argv[])
 		cfg_add_key(&cf, "MediaLibWindowPosX", buf);
 		snprintf(buf, 7, "%d", gml_window_get_pos_y(GMEDIALIB(playlist)));
 		cfg_add_key(&cf, "MediaLibWindowPosY", buf);
+
+		snprintf(buf, 7, "%s",
+		         gml_pl_get_new_playlist_confirm(GMEDIALIB(playlist)) ?
+		         "yes" : "no");
+		cfg_add_key(&cf, "MediaLibPlaylistNewConfirm", buf);
+
 		if (gml_notebook_is_mlib_search_visible(GMEDIALIB(playlist))) {
 			snprintf(buf, 7, "%s",
 			         gml_search_get_column_id_visible(GMEDIALIB(playlist)) ?
